//
// ConfigurationView.h
//
// $Id: //poco/1.4/Util/include/Poco/Util/ConfigurationView.h#1 $
//
// Library: Util
// Package: Configuration
// Module:  ConfigurationView
//
// Definition of the ConfigurationView class.
//
// Copyright (c) 2004-2006, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef Util_ConfigurationView_INCLUDED
#define Util_ConfigurationView_INCLUDED


#include "Poco/Util/Util.h"
#include "Poco/Util/AbstractConfiguration.h"


namespace Poco {
namespace Util {


class Util_API ConfigurationView: public AbstractConfiguration
	/// This configuration implements a "view" into a sub-hierarchy 
	/// of another configuration.
	/// 
	/// For example, given a configuration with the following properties:
	///     config.value1
	///     config.value2
	///     config.sub.value1
	///     config.sub.value2
	/// and a ConfigurationView with the prefix "config", then
	/// the above properties will be available via the view as
	///     value1
	///     value2
	///     sub.value1
	///     sub.value2
	///
	/// A ConfigurationView is most useful in combination with a
	/// LayeredConfiguration.
	///
	/// If a property is not found in the view, it is searched in
	/// the original configuration. Given the above example configuration,
	/// the property named "config.value1" will still be found in the view.
	///
	/// The main reason for this is that placeholder expansion (${property})
	/// still works as expected given a ConfigurationView.
{
public:
	ConfigurationView(const std::string& prefix, AbstractConfiguration* pConfig);
		/// Creates the ConfigurationView. The ConfigurationView does not take
		/// ownership of the passed configuration.

protected:
	bool getRaw(const std::string& key, std::string& value) const;
	void setRaw(const std::string& key, const std::string& value);
	void enumerate(const std::string& key, Keys& range) const;
	void removeRaw(const std::string& key);
	
	std::string translateKey(const std::string& key) const;
	
	~ConfigurationView();

private:
	ConfigurationView(const ConfigurationView&);
	ConfigurationView& operator = (const ConfigurationView&);

	std::string _prefix;
	AbstractConfiguration* _pConfig;
};


} } // namespace Poco::Util


#endif // Util_ConfigurationView_INCLUDED
